// Flutter imports:
import 'package:flutter/material.dart';

// Package imports:
import 'package:flutter_screenutil/flutter_screenutil.dart';

// Project imports:
import 'package:zego_uikit/src/components/audio_video/audio_video_view.dart';
import 'package:zego_uikit/src/components/audio_video_container/picture_in_picture/defines.dart';
import 'package:zego_uikit/src/services/services.dart';

class ZegoLayoutPIPSmallItemList extends StatefulWidget {
  const ZegoLayoutPIPSmallItemList({
    Key? key,
    required this.targetUsers,
    required this.localUser,
    required this.defaultPosition,
    required this.showLocalViewWithVideoOnly,
    required this.onTap,
    this.foregroundBuilder,
    this.backgroundBuilder,
    this.borderRadius,
    this.margin,
    this.size,
  }) : super(key: key);

  final List<ZegoUIKitUser> targetUsers;
  final ZegoUIKitUser localUser;
  final ZegoViewPosition defaultPosition;
  final bool showLocalViewWithVideoOnly;
  final void Function(ZegoUIKitUser user) onTap;
  final AudioVideoViewForegroundBuilder? foregroundBuilder;
  final AudioVideoViewBackgroundBuilder? backgroundBuilder;

  final double? borderRadius;
  final EdgeInsets? margin;
  final Size? size;

  @override
  State<ZegoLayoutPIPSmallItemList> createState() =>
      _ZegoLayoutPIPSmallItemListState();
}

class _ZegoLayoutPIPSmallItemListState
    extends State<ZegoLayoutPIPSmallItemList> {
  late ZegoViewPosition currentPosition;

  Size get designedSize => widget.size ?? Size(139.5.w, 248.0.h);

  double get designedRatio => 9.0 / 16.0;

  @override
  void initState() {
    super.initState();
    currentPosition = widget.defaultPosition;
  }

  @override
  Widget build(BuildContext context) {
    if (!widget.showLocalViewWithVideoOnly) {
      return view(widget.targetUsers);
    }

    return ValueListenableBuilder<bool>(
      valueListenable: ZegoUIKit().getCameraStateNotifier(widget.localUser.id),
      builder: (context, localUserCameraEnabled, _) {
        List<ZegoUIKitUser> targetUsers = List.from(widget.targetUsers);
        if (!localUserCameraEnabled) {
          targetUsers.removeWhere((user) => widget.localUser.id == user.id);
        }
        return view(targetUsers);
      },
    );
  }

  Widget view(List<ZegoUIKitUser> targetUsers) {
    const maxItemCount = 3;

    return calculatePosition(
      child: SizedBox(
        width: designedSize.width,
        height: (maxItemCount - 1) * 16.r + maxItemCount * designedSize.height,
        child: MediaQuery.removePadding(
          context: context,
          removeTop: true,
          removeBottom: true,
          child: ListView.builder(
            shrinkWrap: true,
            physics: const NeverScrollableScrollPhysics(),
            itemCount: targetUsers.length,
            itemBuilder: (context, index) {
              var targetUser = targetUsers[index];
              return Padding(
                padding: EdgeInsets.fromLTRB(
                    0, 0.r, 0, (index == targetUsers.length - 1) ? 0 : 16.r),
                child: SizedBox(
                  width: designedSize.width,
                  height: designedSize.height,
                  child: GestureDetector(
                    onTap: () {
                      widget.onTap(targetUser);
                    },
                    child: AbsorbPointer(
                      child: calculateSize(
                        user: targetUser,
                        child: ZegoAudioVideoView(
                          key: ValueKey(targetUser.id),
                          user: targetUser,
                          borderRadius: widget.borderRadius ?? 18.0.w,
                          backgroundBuilder: widget.backgroundBuilder,
                          foregroundBuilder: widget.foregroundBuilder,
                        ),
                      ),
                    ),
                  ),
                ),
              );
            },
          ),
        ),
      ),
    );
  }

  /// calculates smallView's size based on current screen
  Widget calculateSize({required ZegoUIKitUser? user, required Widget child}) {
    final portraitBaseSize =
        Size(designedSize.height, designedSize.width); // vertical
    final landscapeBaseSize = designedSize; //  horizontal

    if (user == null) {
      Size defaultSize = Size(
          landscapeBaseSize.height * designedRatio, landscapeBaseSize.height);
      return SizedBox.fromSize(size: defaultSize, child: child);
    } else {
      return ValueListenableBuilder<Size>(
        valueListenable: ZegoUIKit().getVideoSizeNotifier(user.id),
        builder: (context, Size size, _) {
          late double width, height;
          if (size.width > size.height) {
            width = portraitBaseSize.width;
            height = portraitBaseSize.width * designedRatio;
          } else {
            width = landscapeBaseSize.height * designedRatio;
            height = landscapeBaseSize.height;
          }

          return SizedBox(width: width, height: height, child: child);
        },
      );
    }
  }

  /// position container, calculates the coordinates based on current position
  Widget calculatePosition({required Widget child}) {
    var paddingSpace = widget.margin ??
        EdgeInsets.only(left: 20.r, top: 50.r, right: 20.r, bottom: 30.r);

    double? left, top, right, bottom;
    switch (currentPosition) {
      case ZegoViewPosition.topLeft:
        left = paddingSpace.left;
        top = paddingSpace.top;
        break;
      case ZegoViewPosition.topRight:
        right = paddingSpace.right;
        top = paddingSpace.top;
        break;
      case ZegoViewPosition.bottomLeft:
        left = paddingSpace.left;
        bottom = paddingSpace.bottom;
        break;
      case ZegoViewPosition.bottomRight:
        right = paddingSpace.right;
        bottom = paddingSpace.bottom;
        break;
    }

    return Positioned(
      left: left,
      top: top,
      right: right,
      bottom: bottom,
      child: child,
    );
  }
}
